/*
    Copyright (C) 2012 George Kiagiadakis <kiagiadakis.george@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef VIDEO_SINK_BIN_H
#define VIDEO_SINK_BIN_H

#include <QGst/Bin>

namespace KTpCallPrivate {

class VideoSinkBin
{
    Q_DISABLE_COPY(VideoSinkBin);
public:
    explicit VideoSinkBin(const QGst::ElementPtr & videoSink);
    virtual ~VideoSinkBin();

    QGst::BinPtr bin() const { return m_bin; }

private:
    QGst::BinPtr m_bin;
};

} // KTpCallPrivate

#endif // VIDEO_SINK_BIN_H
