<?xml version="1.0" encoding="UTF-8"?>
<gui name="callwindow"
     version="1"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd" >

  <MenuBar>
    <Menu name="call" >
      <text>Call</text>
      <Action name="hangup" />
      <Action name="hold" />
    </Menu>
    <Menu name="view">
      <Action name="showMyVideo" />
      <Action name="showDtmf" />
      <Action name="goToSystemTray" />
      <Action name="fullScreen" />
    </Menu>
    <Menu name="window">
      <text>Window</text>
      <Action name="fullScreen" />
    </Menu>
  </MenuBar>
</gui>
