/*
    Copyright (C) 2012 George Kiagiadakis <kiagiadakis.george@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TF_CONTENT_HANDLER_FACTORY_H
#define TF_CONTENT_HANDLER_FACTORY_H

#include "../../libqtf/qtf.h"

namespace KTpCallPrivate {

class TfChannelHandler;
class TfContentHandler;

class TfContentHandlerFactory
{
public:
    typedef TfContentHandlerFactory *(*Constructor)();

    virtual ~TfContentHandlerFactory() {}

    virtual TfContentHandler *createContentHandler(const QTf::ContentPtr & tfContent,
                                                   TfChannelHandler *parent) = 0;
};

} // KTpCallPrivate

#endif
